module elements;

/* This is simple demonstration of how to use expat. This program
     reads an XML document from standard input and writes a line with
     the name of each element to standard output indenting child
     elements by one tab stop more than their parent element.
*/

import std.c.stdio;
import std.c.stdlib;

import expat;

void startElement(void *userData, char *name, char **atts)
{
    int *depthPtr = (int*)userData;
    
    for (int i = 0; i < *depthPtr; i++)
        printf("\t");
        
    printf("%s\n", name);
    *depthPtr += 1;
}

void endElement(void *userData, char *name)
{
    int *depthPtr = (int*)userData;
    *depthPtr -= 1;
}

int main(char[][] args)
{
    XML_Parser parser = XML_ParserCreate(null);
    int depth = 0;
    
    XML_SetUserData(parser, &depth);
    XML_SetElementHandler(parser, &startElement, &endElement);
    
    char buf[256];
    int done;
    
    do 
    {
        size_t len = fread(buf, 1, buf.sizeof, stdin);
        done = len < buf.sizeof;
        
        if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR) 
        {
            fprintf(stderr, "%s at line %d\n",
                XML_ErrorString(XML_GetErrorCode(parser)),
                XML_GetCurrentLineNumber(parser));
            return 1;
        }
    } 
    while (!done);
    
    XML_ParserFree(parser);    
    return 0;
}

